<?php

namespace FiloBlu\Venistar\Esb\Consumer;

use FiloBlu\Esb\Api\Data\MessageInterface;
use FiloBlu\Esb\Api\Data\MessageInterfaceFactory;
use FiloBlu\Esb\Api\Data\ObjectTypeDescriptorInterface;
use FiloBlu\Esb\Api\Data\StatusInterface;
use FiloBlu\Esb\Api\Data\StatusInterfaceFactory;
use FiloBlu\Esb\Core\Exception\NonRecoverableException;
use FiloBlu\Esb\Core\Expression\EvaluatorInterfaceFactory;
use FiloBlu\Esb\Core\Extractor\ObjectTypeFromMessage;
use FiloBlu\Esb\Framework\Consumer\AbstractConsumer;
use FiloBlu\Storelocator\Model\Google\Api;
use Magento\Framework\Exception\NoSuchEntityException;
use FiloBlu\Esb\Model\Saver\OrderSaver;
use Psr\Log\LoggerInterface;
use FiloBlu\Venistar\Helper\Api as ApiHelper;
use FiloBlu\Flow\Model\Connector\Ftp as FlowFtpConnector;

class SendOrderToVenistar extends AbstractConsumer
{
    /**
     * @var ApiHelper
     */
    protected $apiHelper;
    protected $flowFtpConnector;

    public function __construct(
        EvaluatorInterfaceFactory $evaluatorFactory,
        ObjectTypeDescriptorInterface $objectTypeDescriptor,
        MessageInterfaceFactory $messageFactory,
        ObjectTypeFromMessage $objectTypeFromMessage,
        StatusInterfaceFactory $statusFactory,
        LoggerInterface $logger,
        OrderSaver $orderSaver,
        ApiHelper $apiHelper,
        FlowFtpConnector $flowFtpConnector
    ) {
        parent::__construct(
            $evaluatorFactory,
            $objectTypeDescriptor,
            $messageFactory,
            $objectTypeFromMessage,
            $statusFactory,
            $logger
        );
        $this->apiHelper = $apiHelper;
        $this->flowFtpConnector = $flowFtpConnector;
    }

    public function consume(MessageInterface $message): MessageInterface
    {
        $order = $this->loadObject($message);
        $parameters = $this->getConfiguration()->getParameters();
        $this->flowFtpConnector->setConfig($parameters->getDataByKey('ftp'));
        $json = $this->apiHelper->getOrderDataByApi($order->getEntityId(),$parameters->getDataByKey('token'));
        $filename = "order_".$order->getIncrementId()."_".date('YmdHis').".json";
        $filename_path = $this->flowFtpConnector->localDir.DIRECTORY_SEPARATOR.$filename;
        if (!$json) {
            $this->getStatus()->setCode(StatusInterface::ERROR);
            $this->getStatus()->appendOutputData("Can't generate JSON from order {$order->getIncrementId()}");
            return $this->messageFactory->create();
        }

        $fopen = fopen($filename_path, "w");
        fwrite($fopen, $json);
        fclose($fopen);
        $fileUploaded = $this->flowFtpConnector->sendResource($filename_path);

        if (!$fileUploaded) {
            $this->getStatus()->setCode(StatusInterface::ERROR);
            $this->getStatus()->appendOutputData("File upload on FTP failed for order {$order->getIncrementId()}");
            return $this->messageFactory->create();
        }

        $this->getStatus()->setCode(StatusInterface::SUCCESS);
        $this->getStatus()->appendOutputData("File uploaded on FTP successfully for order {$order->getIncrementId()}");
        return $this->messageFactory->create();
    }


    /**
     * @return string
     */
    public function getName(): string
    {
        return 'venistar_order_sender';
    }


    /**
     * @return string
     */
    public function getUniqueName(): string
    {
        return 'FiloBlu_Venistar::order_sender';
    }
}
